const { merge } = require('webpack-merge');
const LoadablePlugin = require('@loadable/webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin');
const CssMinimizerPlugin = require('css-minimizer-webpack-plugin');
const { getProjectPath } = require('../common/utils');
const baseConfig = require('./webpack.base');

module.exports = function (appConfig, extendEnv) {
  return merge(
    baseConfig(appConfig, extendEnv, {
      isServer: false,
      isHotServer: false,
      isProd: true
    }),
    {
      entry: getProjectPath(appConfig.buildClientEntry),
      // 打包的出口
      output: {
        path: getProjectPath(appConfig.buildClientOutput.path),
        publicPath: appConfig.buildClientOutput.publicPath || '/',
        // 入口文件生产的js
        filename: `js/[name].js?v=${extendEnv.version}`,
        // 非入口文件生产的js
        chunkFilename: `js/[name].js?v=${extendEnv.version}`
      },
      resolve: {
        fallback: {
          fs: false
        }
      },
      plugins: [
        new LoadablePlugin({
          filename: appConfig.buildClientLoadableConfig.filename,
          writeToDisk: {
            filename: getProjectPath(appConfig.buildClientLoadableConfig.writeToDiskFilename)
          }
        })
      ],
      optimization: {
        minimizer: [
          // 压缩js
          new TerserPlugin({
            terserOptions: {
              parse: {
                ecma: 8
              },
              compress: {
                ecma: 5,
                warnings: false,
                comparisons: false,
                inline: 2
              },
              mangle: {
                safari10: true
              },
              output: {
                ecma: 5,
                comments: false,
                ascii_only: true
              }
            }
          }),
          // 压缩css
          new CssMinimizerPlugin()
        ],
        splitChunks: {
          chunks: 'initial',
          minSize: 0,
          maxAsyncRequests: 5,
          maxInitialRequests: 3,
          automaticNameDelimiter: '~',
          cacheGroups: {
            common: {
              chunks: 'async',
              name: 'common',
              minChunks: 20,
              priority: 2
            },
            vendor: {
              test: /[\\/]node_modules[\\/]/,
              chunks: 'initial',
              name: 'vendor',
              minChunks: 1,
              priority: 10
            }
          }
        },
        runtimeChunk: {
          name: 'manifest'
        }
      }
    }
  );
};
